`timescale 1ns / 1ns

module Nexys4_tb();
   reg         CLK100MHz=0;
   reg [15:0]  SW=0;		// slide switches
   wire [15:0] LED;		// LEDs
   wire [2:0]  RGB2,RGB1;	// RGB LEDs (R,G,B)
   wire [6:0]  DIG;		// Digit of 7seg, low-active
   wire        DOT;		// Dot of 7seg, low-active
   wire [7:0]  DIGSEL;	// Selector for 7seg, low-active
   reg 	       BTNC=1,BTNU=1,BTNL=1,BTNR=1,BTND=1; // center,up,left,right,down button
   reg 	       BTNCPURST=1;	// CPU reset button, low-active
   reg 	       UARTRX=1, UARTRTS=1; // receive, request to send
   wire        UARTTX, UARTCTS; // transmit, clear to send
   reg 	       tmp=0;
   
   always #5 CLK100MHz = ~CLK100MHz;

   Nexys4_top fpga(CLK100MHz,
		   SW,LED,RGB2,RGB1,
		   DIG,DOT,DIGSEL,
		   BTNC,BTNU,BTNL,BTNR,BTND,
		   UARTRX, UARTTX, UARTCTS, UARTRTS,
		   BTNCPURST
		   );

   initial begin
      $dumpfile("Nexys4_tb.vcd");
      $dumpvars;
      CLK100MHz=1;
      BTNCPURST=1;
      SW=16'hffff;
      #3 tmp=~tmp;
      #100 tmp=~tmp;
      BTNCPURST=0;
      #100 tmp=~tmp;
      BTNCPURST=1;
      #3000 tmp=~tmp;
      #50 SW=16'h0000;
      #50 SW=16'h0001;
      #50 SW=16'h0002;
      #50 SW=16'h0003;
      #50 SW=16'h0004;
      #50 SW=16'h0005;
      #50 SW=16'h0006;
      #50 SW=16'h0007;
      #100 tmp=~tmp;
      $finish;
   end
   
   
endmodule
