/*
 (C) Tomonori Izumi <izumi@ieee.org>, Aug. 2015. All rigts reserved.
 for XC7A100T-1CSG324C on Digilent Nexys4

 Vivado {Kp
 					Ver.2015.08.01a

 ڑFCOM|[gԍ΍Asgp|[g̍폜@B
 Rs[^̃vpeBڍאݒAϐ̐ݒ
 DEVMGR_SHOW_NONPRESENT_DEVICES  1 ɐݒ肷B
 foCX}l[WNA\\̃foCX\
 svȃ|[g폜B

 S25FL128S 128Mbit QSPI
 */

`default_nettype none

module Nexys4_top
  (
   CLK100MHz,
   SW,LED,RGB2,RGB1,
   DIG,DOT,DIGSEL,
   BTNC,BTNU,BTNL,BTNR,BTND,
   UARTRX, UARTTX, UARTCTS, UARTRTS,
   BTNCPURST
   );

   input wire         CLK100MHz;
   input wire [15:0]  SW;		// slide switches
   output wire [15:0] LED;		// LEDs
   output wire [2:0]  RGB2, RGB1;	// RGB LEDs (R,G,B)
   output wire [6:0]  DIG;		// Digit of 7seg, low-active
   output wire 	      DOT;		// Dot of 7seg, low-active
   output wire [7:0]  DIGSEL;	// Selector for 7seg, low-active
   input wire 	      BTNC, BTNU, BTNL, BTNR, BTND;	// center,up,left,right,down button
   input wire 	      BTNCPURST;	// CPU reset button, low-active

   input wire 	      UARTRX, UARTRTS; // receive, request to send
   output wire 	      UARTTX, UARTCTS; // transmit, clear to send

   //----------------------------------------------------------------
   // 

   // acynchronous reset 񓯊Zbg
   wire 	 arstn;
   assign 	 arstn=BTNCPURST;
   
   // clock
   wire 	 clk,CLK100MHz_ibuf;
   IBUFG ibuf_clk(.I(CLK100MHz), .O(CLK100MHz_ibuf));
   BUFG  buf_clk(.I(CLK100MHz_ibuf),.O(clk));

   // setup synchronous reset Zbg
   reg [7:0] 	 rstn_count=255;
   reg 		 rstn_reg=0;
   wire 	 rstn;
   always @ (posedge clk) begin
      if (!arstn) begin
	 rstn_count<=255;
	 rstn_reg<=0;
      end else if (rstn_count!=0) begin
	 rstn_count<=rstn_count-1;
	 rstn_reg<=0;
      end else begin
	 rstn_reg<=1;
      end
   end
   BUFG buf_rstn(.I(rstn_reg), .O(rstn));
   
   //----------------------------------------------------------------
   // general purpose counter ėpJE^
   (* mark_debug = "true" *) reg [31:0] counter=0;
   always @(posedge clk)
     if (!rstn) counter<=0; else counter<=counter+1;

   //----------------------------------------------------------------
   // setup : 16bit slide switches and push buttons
   reg [15:0] 	 SW_reg;
   reg 		 BTNU_reg,BTNL_reg,BTNR_reg,BTND_reg;
   (* mark_debug = "true" *) reg BTNC_reg;
   always @(posedge clk)
     begin // if (counter[19:0]==0) begin // for debouncing/anti-chattering
	SW_reg<=SW;
	BTNC_reg<=BTNC;
	BTNU_reg<=BTNU;
	BTNL_reg<=BTNL;
	BTNR_reg<=BTNR;
	BTND_reg<=BTND;
     end
   
   //----------------------------------------------------------------
   // 16bit LEDs
   reg [15:0] LED_reg;
   assign     LED = LED_reg;

   //----------------------------------------------------------------
   // setup : HEX LEDs and RGB illumination LEDs

   reg [31:0] HEX_reg;
   reg [7:0]  DOTS_reg;
   reg [8:0]  ILUM2_reg,ILUM1_reg; // 9bit R2R1R0 G2G1G0 B2B1B0

   HEXILUMout_module HEXILUM
     (HEX_reg,DOTS_reg,
      ILUM2_reg,ILUM1_reg,
      DIG,DOT,DIGSEL,
      RGB2,RGB1,
      clk);

   //----------------------------------------------------------------
   // setup : UART from/to PC

   (* mark_debug = "true" *) wire [7:0] uartrxdata;
   (* mark_debug = "true" *) wire       uartrxwait,  uarttxvalid;
   (* mark_debug = "true" *) wire [7:0] uarttxdata;
   (* mark_debug = "true" *) wire       uartrxvalid, uarttxwait;
   assign 	 UARTCTS = 0;
   defparam uartrx.PULSEW  = 868; // 100MHz/115200bps=868
   defparam uartrx.PULSEW2 = 434; // PULSEW/2
   UART_RX_module uartrx(UARTRX,uartrxdata,uartrxvalid,uartrxwait,clk,rstn);
   defparam uarttx.PULSEW  = 868; // 100MHz/115200bps=868
   defparam uarttx.PULSEW2 = 434; // PULSEW/2
   UART_TX_module uarttx(uarttxdata,uarttxvalid,uarttxwait,UARTTX,clk,rstn);

   // transmit/receive data counter for debug
   reg [7:0] uartrxcount=0, uarttxcount=0;
   always @(posedge clk)
     if (!rstn) begin
	uartrxcount<=0;
	uarttxcount<=0;
     end else begin
	if (uartrxvalid&~uartrxwait) begin
	   uartrxcount<=uartrxcount+1;
	end
	if (uarttxvalid&~uarttxwait) begin
	   uarttxcount<=uarttxcount+1;
	end
     end
   
   // simple loopback for UART sample 啶̏ϊ
   reg [7:0]  uartlbdata_reg=0;
   reg 	      uartlbvalid_reg=0;
   always @(posedge clk)
     if (!rstn) begin
	uartlbvalid_reg<=0;
     end else begin
	if (uartrxvalid&~uartlbvalid_reg) begin // receive
	   if (uartrxdata>=8'h41 && uartrxdata<=8'h5a)
	     uartlbdata_reg<=uartrxdata+32; // upper -> lower
	   else if (uartrxdata>=8'h61 && uartrxdata<=8'h7a)
	     uartlbdata_reg<=uartrxdata-32; // lower -> upper
	   else 
	     uartlbdata_reg<=uartrxdata;
	   uartlbvalid_reg<=1;
	end else if (uartlbvalid_reg&~uarttxwait) begin // send
	   uartlbvalid_reg<=0;
	end
     end

   assign uartrxwait=~uartlbvalid_reg;
   assign uarttxvalid=uartlbvalid_reg;
   assign uarttxdata=uartlbdata_reg;

   //----------------------------------------------------------------
   // LEDs

   always @(posedge clk) 
     case (SW_reg)
       16'h0000 : 
	 LED_reg<=16'h0000;
       16'h0001 : 
	 LED_reg<=16'h0001;
       16'h0002 : 
	 LED_reg<=16'h0003;
       16'h0003 : 
	 LED_reg<=16'h0007;
       16'h0004 : 
	 LED_reg<=16'h000f;
       16'h0005 : 
	 LED_reg<=16'h001f;
       16'h0006 : 
	 LED_reg<=16'h003f;
       16'h0007 : 
	 LED_reg<=16'h007f;
       16'h0008 : 
	 LED_reg<=16'h00ff;
       default :
	 LED_reg<=counter[31:16];
     endcase
   

   //----------------------------------------------------------------
   // ILUM LEDs

   always @(posedge clk) begin
      if (BTNC_reg) begin
	 ILUM1_reg<= counter[31:23];
	 ILUM2_reg<= counter[22:14];
      end else begin
	 ILUM1_reg<= {2'b0,uarttxcount[2],
		      2'b0,uarttxcount[1],
		      2'b0,uarttxcount[0]};
	 ILUM2_reg<= {2'b0,uartrxcount[2],
		      2'b0,uartrxcount[1],
		      2'b0,uartrxcount[0]};
      end
   end
   
   //----------------------------------------------------------------
   // 7seg LEDs

   always @(posedge clk) begin
      if (BTNC_reg) begin
	 HEX_reg<={uarttxcount,uarttxdata,uartrxcount,uartrxdata};
	 DOTS_reg<={uarttxvalid,uarttxvalid,uarttxwait,uarttxwait,
		    uartrxvalid,uartrxvalid,uartrxwait,uartrxwait};
      end else begin
	 HEX_reg<={counter[31:16],SW_reg};
	 DOTS_reg<={BTNL_reg,BTNU_reg,BTND_reg,BTNR_reg,
		    BTNL_reg,BTNU_reg,BTND_reg,BTNR_reg};
      end
   end


endmodule // Nexys4_top

`default_nettype wire
