// (C) Tomonori Izumi <izumi@ieee.org>, Mar. 2014. All rigts reserved.
// for XC7A100T-1CSG324C on Digilent Nexys4

//================================================================
// modulation by switching for RGB LEDs and digits of 7seg LEDs
//
// HEX7    HEX6    HEX5    HEX4    HEX3    HEX2    HEX1    HEX0   
//  --      --      --      --      --      --      --      --   
// |  |    |  |    |  |    |  |    |  |    |  |    |  |    |  |  
//  --      --      --      --      --      --      --      --   
// |  |    |  |    |  |    |  |    |  |    |  |    |  |    |  |  
//  --  .   --  .   --  .   --  .   --  .   --  .   --  .   --  .
//   DOT7    DOT6    DOT5    DOT4    DOT3    DOT2    DOT1    DOT0
//
// ILUM2  ILUM1
// +---+  +---+
// |RGB|  |RGB|     9bit [R2 R1 R0 G2 G1 G0 B2 B1 B0]
// |LED|  |LED|
// +---+  +---+

`default_nettype none

module HEXILUMout_module
  (HEX,DOTS,
   ILUM2,ILUM1,
   DIGo,DOTo,DIGSELo,
   RGB2o,RGB1o,
   clk);
   
   input wire [31:0]  HEX;
   input wire [7:0]   DOTS;
   input wire [8:0]   ILUM2,ILUM1;
   output wire [6:0] DIGo;
   output wire 	     DOTo;
   output wire [7:0] DIGSELo; // low-active
   output wire [2:0] RGB2o,RGB1o;
   input wire 	     clk;

   reg [2:0] 	RGB2_reg=0, RGB1_reg=0;
   reg [3:0] 	HEX_reg=0;
   reg 		DOT_reg=0;
   reg [7:0] 	DIGSEL_reg=0;
   reg [23:0] 	LEDcount=0;

   function [6:0] hex7seg;
      input [3:0] data;
      begin
	 case (data)           // 6543210
	   4'b0000 : hex7seg = 7'b0111111;
	   4'b0001 : hex7seg = 7'b0000110;
	   4'b0010 : hex7seg = 7'b1011011;
	   4'b0011 : hex7seg = 7'b1001111;
	   4'b0100 : hex7seg = 7'b1100110;
	   4'b0101 : hex7seg = 7'b1101101;
	   4'b0110 : hex7seg = 7'b1111101;
	   4'b0111 : hex7seg = 7'b0000111;
	   4'b1000 : hex7seg = 7'b1111111;
	   4'b1001 : hex7seg = 7'b1101111;
	   4'b1010 : hex7seg = 7'b1110111;
	   4'b1011 : hex7seg = 7'b1111100;
	   4'b1100 : hex7seg = 7'b1011000;
	   4'b1101 : hex7seg = 7'b1011110;
	   4'b1110 : hex7seg = 7'b1111001;
	   4'b1111 : hex7seg = 7'b1110001;
	 endcase // case(data)
      end
   endfunction // hex7seg

   assign 	DIGo = ~hex7seg(HEX_reg);
   assign 	DOTo = ~DOT_reg;
   assign 	DIGSELo = ~DIGSEL_reg;
   assign 	RGB2o = RGB2_reg;
   assign 	RGB1o = RGB1_reg;

   //----------------------------------------------------------------
   // modulation by switching for RGB LEDs and digits of 7seg LEDs
   always @(posedge clk) 
     begin
	LEDcount<=LEDcount+1;

	RGB1_reg 
	  <= {(LEDcount[19:17]<ILUM1[8:6])?1'b1:1'b0,
	      (LEDcount[19:17]<ILUM1[5:3])?1'b1:1'b0,
	      (LEDcount[19:17]<ILUM1[2:0])?1'b1:1'b0};
	RGB2_reg 
	  <= {(LEDcount[19:17]<ILUM2[8:6])?1'b1:1'b0,
	      (LEDcount[19:17]<ILUM2[5:3])?1'b1:1'b0,
	      (LEDcount[19:17]<ILUM2[2:0])?1'b1:1'b0};

	case (LEDcount[19:17])
	  3'b000 : begin HEX_reg<=HEX[ 3: 0];DOT_reg<=DOTS[0];DIGSEL_reg<=8'h01; end
	  3'b001 : begin HEX_reg<=HEX[ 7: 4];DOT_reg<=DOTS[1];DIGSEL_reg<=8'h02; end
	  3'b010 : begin HEX_reg<=HEX[11: 8];DOT_reg<=DOTS[2];DIGSEL_reg<=8'h04; end
	  3'b011 : begin HEX_reg<=HEX[15:12];DOT_reg<=DOTS[3];DIGSEL_reg<=8'h08; end
	  3'b100 : begin HEX_reg<=HEX[19:16];DOT_reg<=DOTS[4];DIGSEL_reg<=8'h10; end
	  3'b101 : begin HEX_reg<=HEX[23:20];DOT_reg<=DOTS[5];DIGSEL_reg<=8'h20; end
	  3'b110 : begin HEX_reg<=HEX[27:24];DOT_reg<=DOTS[6];DIGSEL_reg<=8'h40; end
	  3'b111 : begin HEX_reg<=HEX[31:28];DOT_reg<=DOTS[7];DIGSEL_reg<=8'h80; end
	endcase
     end

endmodule // HEXILUMout_module

`default_nettype wire
